import { AsyncServiceImplementationError } from "../di/errors/AsyncServiceImplementationError.js";
import { getNameOfServiceType } from "../helpers/getNameOfServiceType.js";
import { Debug } from "../debug.js";
import { BaseServiceImplementationResolver } from "./BaseServiceImplementationResolver.js";
import { RegistryServiceImplementationResolver } from "./RegistryServiceImplementationResolver.js";
const NOT_LOADED = Symbol("NOT_LOADED");
export class AsyncServiceImplementationResolver extends BaseServiceImplementationResolver {
    loader;
    get name() {
        if (this.implementation === NOT_LOADED) {
            //"lazy(?)"
            return Debug.errors.serviceNotLoaded(this.loader);
        }
        return getNameOfServiceType(this.implementation);
    }
    implementation;
    promise;
    constructor(loader) {
        super();
        this.loader = loader;
        this.implementation = NOT_LOADED;
        this.promise = null;
    }
    *resolve(internalGetService, descriptor, requestedBy, mode) {
        if (mode === "sync" && descriptor.dry) {
            return yield* new RegistryServiceImplementationResolver(null).resolve(internalGetService, descriptor, requestedBy, mode);
        }
        if (!this.isLoaded() || mode === "sync") {
            yield this.load();
        }
        return yield* this.getImplementation().resolve(internalGetService, descriptor, requestedBy, mode);
    }
    isLoaded() {
        return this.implementation !== NOT_LOADED;
    }
    getImplementation() {
        if (this.implementation === NOT_LOADED) {
            throw new AsyncServiceImplementationError();
        }
        return this.implementation;
    }
    async load() {
        if (this.implementation !== NOT_LOADED) {
            return this.implementation;
        }
        if (!this.promise) {
            this.promise = this.loader()
                .then((implementation) => {
                this.implementation = new RegistryServiceImplementationResolver(implementation);
                return this.implementation;
            })
                .finally(() => {
                this.promise = null;
            });
        }
        return this.promise;
    }
}
//# sourceMappingURL=AsyncServiceImplementationResolver.js.map